/*  lircrc-config pretends to be a GUI for LIRC's rc files.
    Copyright (C) 2001 Manuel Estrada Sainz <ranty@atdot.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <glib.h>
#include "comments.h"
#include "parser.h"

static GHashTable *main_hash = NULL;
static GHashTable *mode_hash = NULL;
static GHashTable *entry_text_pos = NULL;

void init_comments (void)
{
	main_hash = g_hash_table_new (NULL, NULL);
	mode_hash = g_hash_table_new (g_str_hash, g_str_equal);
	entry_text_pos = g_hash_table_new(NULL, NULL);
}

gchar **new_comments (void *element, gint size)
{
	char **comments = g_hash_table_lookup(main_hash, element);
	
	if (comments != NULL)
		g_free(comments);
	
	comments = g_new0(char *, size);
	g_hash_table_insert(main_hash, element, comments);

	return comments;
}
gchar **get_comments (void *element)
{
	char **comments = g_hash_table_lookup(main_hash, element);
	g_assert(comments != NULL);
	return comments;
}

void kill_comments (void *element)
{
	char **comments = g_hash_table_lookup(main_hash, element);
	g_assert(comments != NULL);
	g_hash_table_remove(main_hash, element);
	g_free(comments);
}
gchar **new_mode_comments (char *mode)
{
	char **comments = g_hash_table_lookup(mode_hash, mode);

	/* get our own copy of mode so we don't have to care for what the
	 * caller does with his own */
	mode = g_strdup(mode);
	
	if (comments != NULL)
		g_free(comments);
	
	comments = g_new0(char *, lirc_mode_com_TOTAL);
	g_hash_table_insert(mode_hash, mode, comments);

	return comments;
}
void rename_mode_comments (char *mode, char *new_mode)
{
	gpointer comments;
	gpointer orig_key;
	gboolean found;

	found = g_hash_table_lookup_extended(mode_hash, mode,
					     &orig_key, &comments);

	g_assert(found);
	g_hash_table_remove(mode_hash, mode);
	g_free(orig_key);

#warning what to do if new_mode already exists?
	g_return_if_fail(g_hash_table_lookup(mode_hash, new_mode) == NULL);

	/* get our own copy of mode so we don't have to care for what the
	 * caller does with his own */
	new_mode = g_strdup(new_mode);
	
	g_hash_table_insert(mode_hash, new_mode, comments);
}

gchar **get_mode_comments (char *mode)
{
	char **comments = g_hash_table_lookup(mode_hash, mode);
	g_assert(comments != NULL);
	return comments;
}

void kill_mode_comments (char *mode)
{
	gpointer comments;
	gpointer orig_key;
	gboolean found;

	found = g_hash_table_lookup_extended(mode_hash, mode,
					     &orig_key, &comments);

	g_assert(found);
	g_hash_table_remove(mode_hash, mode);
	g_free(comments);
	g_free(orig_key);
}

void set_entry_text_pos(void *element, guint16 start, guint16 end)
{
	guint32 value = start<<16 | end;
	g_hash_table_insert(entry_text_pos, element, GINT_TO_POINTER(value));
}
gint get_entry_text_pos(void *element, guint16 *end)
{
	gboolean found;
	gpointer value;
	gpointer orig_element;

	found = g_hash_table_lookup_extended(entry_text_pos, element,
					     &orig_element, &value);

	g_return_val_if_fail(orig_element == element, -1);

	if(found){
		if(end)
			*end = GPOINTER_TO_INT(value) & 0xFFFF;
		return GPOINTER_TO_INT(value)>>16 & 0xFFFF;
	}else 
		return -1;
}
void reset_entry_text_pos(void)
{
	g_hash_table_destroy(entry_text_pos);
	entry_text_pos = g_hash_table_new(NULL, NULL);
}
