/*  lircrc-config pretends to be a GUI for LIRC's rc files.
    Copyright (C) 2001 Manuel Estrada Sainz <ranty@atdot.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <string.h>
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include "file-dialog.h"
#include "parser.h"
#include "interface.h"

static GtkWidget *file_sel = NULL;
static gboolean selection_ok = FALSE;
static GtkCTree *config_ctree;

static gchar *last_filename = NULL;

static gint delete_event (GtkWidget *widget)
{
	gtk_widget_hide(GTK_WIDGET(widget));
	return TRUE;
}
static void file_selected_cancel(GtkWidget *button, GtkWidget *file_sel)
{
	gtk_widget_hide(GTK_WIDGET(file_sel));
}
static void file_selected_callback(GtkWidget *button, GtkWidget *file_sel)
{
	selection_ok = TRUE;
	gtk_widget_hide(GTK_WIDGET(file_sel));
}
const gchar * select_file(void)
{
	g_return_val_if_fail(file_sel != NULL, NULL);

	selection_ok = FALSE;
	gtk_widget_show(GTK_WIDGET(file_sel));
	if (!selection_ok)
		return NULL;
	return gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_sel));
}
static void create_file_dialog(void)
{
	file_sel=gtk_file_selection_new(_("Select a file"));
	gtk_window_set_modal(GTK_WINDOW(file_sel), TRUE);
	gtk_signal_connect (GTK_OBJECT (file_sel), "delete_event",
			    GTK_SIGNAL_FUNC (delete_event), NULL);
	gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(
				file_sel)->ok_button), "clicked",
				GTK_SIGNAL_FUNC (file_selected_callback),
				file_sel);
	gtk_signal_connect (GTK_OBJECT(file_sel), "show",
				GTK_SIGNAL_FUNC (gtk_main),
				file_sel);
	gtk_signal_connect (GTK_OBJECT(file_sel), "hide",
				GTK_SIGNAL_FUNC (gtk_main_quit),
				file_sel);
	gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(file_sel)
				->cancel_button), "clicked",
				GTK_SIGNAL_FUNC (file_selected_cancel),
				file_sel);
}

static void remember_filename(const gchar *filename)
{
	if (filename == last_filename)
		return;
	g_free((gchar *)last_filename);
	last_filename = g_strdup(filename);
}

void open_config_file(const gchar *file_name)
{
	struct lirc_config *lirc_config;

	if(file_name == NULL)
		file_name = last_filename;
	
	if(file_name == NULL)
		file_name = select_file();

	if(file_name == NULL)
		return;

	remember_filename (file_name);

	lirc_config = parse_config_file (file_name);
	
	if(!lirc_config)
		return;

	fill_CTree(lirc_config);
}
void open_config_file_cb(void)
{
	const gchar *file_name;
	file_name = select_file();

	if(file_name == NULL)
		return;

	open_config_file(file_name);
}

void revert_config_file_cb(void)
{
	open_config_file(NULL);
}

void new_config_file(void)
{
	last_filename = NULL;
	fill_CTree(g_new0(struct lirc_config, 1));
}

void save_config_file(const gchar *file_name)
{
	struct lirc_config *lirc_config =
		gtk_object_get_data(GTK_OBJECT(config_ctree),
				    "lirc_config");
	GString *str;
	FILE *file;

	if(file_name == NULL)
		file_name = last_filename;

	if(file_name == NULL)
		file_name = select_file();

	if (lirc_config == NULL || file_name == NULL)
		return;

	if((file = fopen(file_name, "w")) == NULL){
		g_message("Couldn't open file: %s", file_name);
		return;
	}

	remember_filename (file_name);

	/* make sure that all changes are applyed to the data
	 * structures */
	interface_apply_changes();

	str = g_string_new("");
	config_2_g_string(str, lirc_config);
	fputs(str->str, file);

	fclose(file);
}
void save_config_file_as_cb(void)
{
	const gchar *file_name = select_file();
	if (file_name != NULL)
		save_config_file(file_name);
}
void save_config_file_cb(void)
{
	save_config_file(NULL);
}
void init_file_read_write(GladeXML *xml)
{
	create_file_dialog();
	config_ctree = GTK_CTREE(
		glade_xml_get_widget(xml, "config_ctree"));
	glade_xml_signal_connect(xml, "on_open_config_file",
				 open_config_file_cb);
	glade_xml_signal_connect(xml, "on_revert_config_file",
				 revert_config_file_cb);
	glade_xml_signal_connect(xml, "on_save_config_file",
				 save_config_file_cb);
	glade_xml_signal_connect(xml, "on_save_config_file_as",
				 save_config_file_as_cb);
	glade_xml_signal_connect(xml, "on_new_config_file", new_config_file);

}
