/*  lircrc-config pretends to be a GUI for LIRC's rc files.
    Copyright (C) 2001 Manuel Estrada Sainz <ranty@atdot.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <glade/glade.h>
#include <gnome.h>
#include "interface.h"
#include "interface-text.h"
#include "gtk-helpers.h"
#include "parser.h"

static GtkWidget *config_ctree;

static GtkEditable *text_editor;

static guint activity_signals[4] = {0};

static GdkFont *font;

static gboolean jump_to_current_entry_text_pos(void);

static gboolean enable_activity_signals(void)
{
	gint i=0;
	while(activity_signals[i])
		gtk_signal_handler_unblock (GTK_OBJECT(config_ctree),
					    activity_signals[i++]);
	/* return FALSE so we only get called once when used with
	 * gtk_idle_add */
	return FALSE;
}

static gboolean disable_activity_signals(void)
{
	gint i=0;
	while(activity_signals[i])
		gtk_signal_handler_block (GTK_OBJECT(config_ctree),
					  activity_signals[i++]);
	/* return FALSE so we only get called once when used with
	 * gtk_idle_add */
	return FALSE;
}

static gboolean reload_config(void)
{
	struct lirc_config *lirc_config;
	GString *str;

	lirc_config = gtk_object_get_data(GTK_OBJECT(config_ctree),
					  "lirc_config");

	/* make sure that all changes are applyed to the data
	 * structures */
	interface_apply_changes();
	
	str = g_string_new("");
	config_2_g_string(str, lirc_config);
	
	gtk_editable_set_text(text_editor, str->str, font);
	widget_changed_cancel(GTK_WIDGET(text_editor));
	
	g_string_free(str, TRUE);
	/* return FALSE so we only get called once when used with
	 * gtk_idle_add */
	return FALSE;
}

static void reparse_config(void)
{
	struct lirc_config *lirc_config;
	gchar *str;

	if(!widget_changed_cancel(GTK_WIDGET(text_editor)))
		return;
	
	widget_changed_cancel(GTK_WIDGET(text_editor));

	str = gtk_editable_get_chars(text_editor, 0, -1);

	lirc_config = parse_config_text(str);

	disable_activity_signals();
	if(lirc_config)
		fill_CTree(lirc_config);
	else
		reload_config();
	enable_activity_signals();

	g_free(str);
}


static gboolean map (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	/* we will be visible, owr notebook tab has been selected */

	reload_config();
	jump_to_current_entry_text_pos();
	enable_activity_signals();

	return TRUE;
}
static gboolean unmap (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	/* we will not be visible, another notebook tab has been
	 * selected */

	reparse_config();
	disable_activity_signals();

	return TRUE;
}

static void ctree_node_select(GtkCTree *ctree,
			      GList *node,
			      gint column,
			      gpointer arg)
{
	int is_leaf;
	struct lirc_config *config =
		gtk_object_get_data(GTK_OBJECT(config_ctree), "lirc_config");
	struct lirc_config_entry *entry =
		gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(node));

	g_return_if_fail(config != NULL);

	if(entry == NULL){
		/* the ctree is probably refilling */
		disable_activity_signals();
		gtk_idle_add ((GtkFunction)reload_config, NULL);
		gtk_idle_add ((GtkFunction)enable_activity_signals, NULL);
		return;
	}

	gtk_ctree_get_node_info (ctree,
				 GTK_CTREE_NODE(node),
				 NULL, NULL,
				 NULL, NULL, NULL, NULL,
				 &is_leaf,
				 NULL);
	if(is_leaf){
		guint16 end;
		gint start = get_entry_text_pos(entry, &end);
		gint size = gtk_text_get_length(GTK_TEXT(text_editor));
		if(start >= 0){
			g_return_if_fail(start <= size);
			g_return_if_fail(end <= size);
			if(end > 0)
				/* try to center the entry */
				gtk_editable_set_position(text_editor, end);
			gtk_editable_set_position(text_editor, start);
		} else{
		}
	}
			
}
static gboolean jump_to_current_entry_text_pos(void)
{
	GList *selection = GTK_CLIST(config_ctree)->selection;

	if(selection)
		ctree_node_select(GTK_CTREE(config_ctree), selection->data,
				  0, NULL);

	/* return FALSE so we only get called once when used with
	 * gtk_idle_add */
	return FALSE;
}

static void ctree_node_unselect(GtkCTree *ctree,
			      GList *node,
			      gint column,
			      gpointer arg)
{
	struct lirc_config *config =
		gtk_object_get_data(GTK_OBJECT(config_ctree), "lirc_config");
	struct lirc_config_entry *entry =
		gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(node));

	g_return_if_fail(config != NULL);

	if(entry == NULL){
		/* this node is probably been removed, maybe the ctree
		 * is refilling */
		disable_activity_signals();
		gtk_idle_add ((GtkFunction)reload_config, NULL);
		gtk_idle_add ((GtkFunction)enable_activity_signals, NULL);
		gtk_idle_add ((GtkFunction)jump_to_current_entry_text_pos,
			      NULL);
		return;
	}
}

void init_interface_text(GladeXML *xml)
{
//	GtkWidget *notebook;
	int sig_i;

	/* get an apropriate font for the text */
	font = gdk_font_load (
		"-dec-terminal-medium-r-normal-*-*-140-*-*-c-*-iso8859-1");
	if(font == NULL)
		font = gdk_font_load ("-*-*-*-*-*--*-*-*-*-c-*-iso8859-1");

	/* get required widget pointers */
	config_ctree = glade_xml_get_widget(xml, "config_ctree");
	text_editor = GTK_EDITABLE(
		glade_xml_get_widget(xml, "config-text-editor"));
	connect_changed_signal(GTK_WIDGET(text_editor));

	gtk_text_set_word_wrap(GTK_TEXT(text_editor), FALSE);
	gtk_text_set_line_wrap(GTK_TEXT(text_editor), FALSE);

//	notebook = glade_xml_get_widget(xml, "notebook");

	gtk_signal_connect(GTK_OBJECT(text_editor),
			   "map",
			   (GtkSignalFunc)map,
			   NULL);
	gtk_signal_connect(GTK_OBJECT(text_editor),
			   "unmap",
			   (GtkSignalFunc)unmap,
			   NULL);

	/* Do things when the user selects a different entry */
	sig_i=0;
	activity_signals[sig_i++] =
		gtk_signal_connect(GTK_OBJECT(config_ctree),
				   "tree-unselect-row",
				   ctree_node_unselect,
				   NULL);

	activity_signals[sig_i++] =
		gtk_signal_connect(GTK_OBJECT(config_ctree),
				   "tree-select-row",
				   ctree_node_select,
				   NULL);

	disable_activity_signals();
}
