/*  lircrc-config pretends to be a GUI for LIRC's rc files.
    Copyright (C) 2001 Manuel Estrada Sainz <ranty@atdot.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <glade/glade.h>
#include <gnome.h>
#include "interface.h"
#include "interface-lircd.h"
#include "interface-text.h"
#include "interface-lists.h"
#include "parser.h"
#include "file-dialog.h"
#include "gtk-helpers.h"

static GtkWidget *editor_main_table;
static GtkCTree *config_ctree;

static GtkEntry *mode_entry;
static GtkEntry *prog_entry;
static GtkEntry *remote_entry;
static GtkEntry *button_entry;
static GtkEntry *config_entry;
static GtkEntry *tag_entry;

static GtkCombo *mode_combo;
static GtkCombo *prog_combo;
static GtkCombo *remote_combo;
static GtkCombo *button_combo;
static GtkCombo *config_combo;

static GtkSpinButton *repeat_spinbutton;

static GtkToggleButton *once_checkbutton;
static GtkToggleButton *quit_checkbutton;
static GtkToggleButton *mode_checkbutton;
static GtkToggleButton *startup_mode_checkbutton;

static GtkList *button_list;
static GtkList *config_list;

static GtkWidget *about_dialog;

static void show_about_dialog(void)
{
	gtk_widget_show(about_dialog);
}

static void load_mode(GtkCTree *ctree, GtkCTreeNode *node)
{
}

static void unload_mode(GtkCTree *ctree, GtkCTreeNode *node)
{
}

static void load_entry(GtkCTree *ctree, GtkCTreeNode *node)
{
	struct lirc_config_entry *entry =
		gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(node));
	char **comments = entry ? get_comments(entry) : NULL;
	g_return_if_fail(entry!=NULL);
	g_return_if_fail(comments!=NULL);

	gtk_widget_set_sensitive(editor_main_table, TRUE);

	gtk_entry_set_text(prog_entry, entry->prog);

	if(entry->change_mode)
		gtk_entry_set_text(mode_entry, entry->change_mode);
	else
		gtk_entry_set_text(mode_entry, "");

	if(comments[lirc_com_tag])
		gtk_entry_set_text(tag_entry, comments[lirc_com_tag]);
	else
		gtk_entry_set_text(tag_entry, "");

	if(entry->code){
		gtk_widget_set_sensitive(GTK_WIDGET(remote_combo), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(button_combo), TRUE);
		if (entry->code->remote)
			gtk_entry_set_text(remote_entry, entry->code->remote);
		else
			gtk_entry_set_text(remote_entry, "*");
		if(entry->code->button)
			gtk_entry_set_text(button_entry, entry->code->button);
		else
			gtk_entry_set_text(button_entry, "*");
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(remote_combo), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(button_combo), FALSE);
	}

	gtk_list_clear_items(button_list, 0, -1);
	gtk_object_set_data(GTK_OBJECT(button_list),
			    "lirc_code", &entry->code);
	entry->next_code = entry->code;
	while(entry->next_code){
		GtkWidget *item;
		item = gtk_list_item_new_with_label(
			entry->next_code->button);
		gtk_widget_show (item);
		gtk_container_add(GTK_CONTAINER(button_list), item);
		gtk_object_set_data(GTK_OBJECT(item),
				    "lirc_code", entry->next_code);
		entry->next_code=entry->next_code->next;
	}

	if (entry->config)
		gtk_entry_set_text(config_entry, entry->config->string);
	else
		gtk_entry_set_text(config_entry, "");

	gtk_list_clear_items(config_list, 0, -1);
	gtk_object_set_data(GTK_OBJECT(config_list),
			    "lirc_config", &entry->config);

	entry->next_config = entry->config;
	while(entry->next_config){
		GtkWidget *item;
		item = gtk_list_item_new_with_label(
			entry->next_config->string);
		gtk_widget_show (item);
		gtk_container_add(GTK_CONTAINER(config_list), item);
		gtk_object_set_data(GTK_OBJECT(item),
				    "lirc_config", entry->next_config);
		entry->next_config=entry->next_config->next;
	}

	gtk_spin_button_set_value(repeat_spinbutton, entry->rep);

	gtk_toggle_button_set_active(once_checkbutton, entry->flags & once);
	gtk_toggle_button_set_active(quit_checkbutton, entry->flags & quit);
	gtk_toggle_button_set_active(mode_checkbutton, entry->flags & mode);
	gtk_toggle_button_set_active(startup_mode_checkbutton,
				     entry->flags & startup_mode);

	/* reset changed status */
	gtk_container_foreach_recursive (GTK_WIDGET(editor_main_table),
					 (GtkCallback)widget_changed_cancel,
					 NULL);
}

static void unload_entry(GtkCTree *ctree, GtkCTreeNode *node)
{
	struct lirc_config_entry *entry =
		gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(node));
	char **comments = entry ? get_comments(entry) : NULL;
	
	if(entry==NULL)
		/* this node is probably been removed */
		return;

	g_return_if_fail(comments!=NULL);

	gtk_widget_set_sensitive(editor_main_table, FALSE);

	/* clear widgets */
	widget_changed_apply(GTK_WIDGET(prog_entry), &entry->prog);
	widget_changed_apply(GTK_WIDGET(mode_entry), &entry->change_mode);
	widget_changed_apply(GTK_WIDGET(tag_entry), &comments[lirc_com_tag]);
	

	/* only update the lirc_code on entry change if the
           button_list is invisible */
	if(entry->code && !GTK_WIDGET_VISIBLE(button_list)){
		widget_changed_apply(GTK_WIDGET(remote_entry),
				     &entry->code->remote);
		widget_changed_apply(GTK_WIDGET(button_entry),
				     &entry->code->button);
	} else {
		gtk_entry_set_text(remote_entry, "");
		gtk_entry_set_text(button_entry, "");
	}

	gtk_list_clear_items(button_list, 0, -1);

	/* only update the lirc_config on entry change if the
           config_list is invisible */
	if(entry->config && !GTK_WIDGET_VISIBLE(config_list))
		widget_changed_apply(GTK_WIDGET(config_entry),
				     &entry->config->string);
	else
		gtk_entry_set_text(config_entry, "");

	gtk_list_clear_items(config_list, 0, -1);

	widget_changed_apply(GTK_WIDGET(repeat_spinbutton), &entry->rep);

	/* clear known flags */
	entry->flags &= ~(once|quit|mode|startup_mode);

	/* apply flags to the entry */
	if(gtk_toggle_button_get_active(once_checkbutton))
		entry->flags |= once;
	if(gtk_toggle_button_get_active(quit_checkbutton))
		entry->flags |= quit;
	if(gtk_toggle_button_get_active(mode_checkbutton))
		entry->flags |= mode;
	if(gtk_toggle_button_get_active(startup_mode_checkbutton))
		entry->flags |= startup_mode;

	/* clear flags on the interface */
	gtk_toggle_button_set_active(once_checkbutton, FALSE);
	gtk_toggle_button_set_active(quit_checkbutton, FALSE);
	gtk_toggle_button_set_active(mode_checkbutton, FALSE);
	gtk_toggle_button_set_active(startup_mode_checkbutton, FALSE);

	/* update the text on the CTree */
	gtk_ctree_node_set_text (ctree, node, 0, entry->prog);

	if(comments[lirc_com_tag])
		gtk_ctree_node_set_text(ctree, node, 1,
					comments[lirc_com_tag]);
	else
		gtk_ctree_node_set_text(ctree, node, 1, "");
	
	if(entry->code && entry->code->remote)
		gtk_ctree_node_set_text(ctree, node, 2, entry->code->remote);
	else
		gtk_ctree_node_set_text(ctree, node, 2, "*");
	
	if(entry->code && entry->code->button)
		gtk_ctree_node_set_text(ctree, node, 3, entry->code->button);
	else
		gtk_ctree_node_set_text(ctree, node, 3, "*");
	
	if(entry->config)
		gtk_ctree_node_set_text(ctree, node, 4, entry->config->string);
}

static void ctree_node_select(GtkCTree *ctree,
			      GList *node,
			      gint column,
			      gpointer arg)
{
	int is_leaf;

	gtk_ctree_get_node_info (ctree,
				 GTK_CTREE_NODE(node),
				 NULL, NULL,
				 NULL, NULL, NULL, NULL,
				 &is_leaf,
				 NULL);
	if(is_leaf)
		load_entry(ctree, GTK_CTREE_NODE(node));
	else
		load_mode(ctree, GTK_CTREE_NODE(node));
			
}

static void ctree_node_unselect(GtkCTree *ctree,
				GList *node,
				gint column,
				gpointer arg)
{
	int is_leaf;

	gtk_ctree_get_node_info (ctree,
				 GTK_CTREE_NODE(node),
				 NULL, NULL,
				 NULL, NULL, NULL, NULL,
				 &is_leaf,
				 NULL);
	if(is_leaf)
		unload_entry(ctree, GTK_CTREE_NODE(node));
	else
		unload_mode(ctree, GTK_CTREE_NODE(node));
			
}

void remove_node(GtkCTree *ctree, GtkCTreeNode *node, gpointer data)
{
	gtk_ctree_remove_node(ctree,node);
}
void delete_item(void)
{
	GList *selection = GTK_CLIST(config_ctree)->selection;
	GtkCTreeNode *node = selection ? selection->data: NULL;
	GtkCTreeNode *parent;

	struct lirc_config *lirc_config =
		gtk_object_get_data(GTK_OBJECT(config_ctree), "lirc_config");
	struct lirc_config_entry *cur_entry =
		gtk_ctree_node_get_row_data(config_ctree, node);
	int is_leaf;

	if(!node || !lirc_config)
		return;

	gtk_ctree_get_node_info (config_ctree,
				 GTK_CTREE_NODE(node),
				 NULL, NULL,
				 NULL, NULL, NULL, NULL,
				 &is_leaf,
				 NULL);

	if(!is_leaf) /* this is a mode node */
		return;

	/* remove the entry pointer from the node so other code knows
	 * that we are deleting it when it gets unselected */
	gtk_ctree_node_set_row_data(config_ctree, node, NULL);

	if(GTK_CTREE_NODE_NEXT(node))
		gtk_ctree_select(config_ctree, GTK_CTREE_NODE_NEXT(node));


	parent = GTK_CTREE_ROW(node)->parent;

	gtk_ctree_remove_node(config_ctree, node);
	if(parent && !GTK_CTREE_ROW(parent)->children)
		gtk_ctree_remove_node(config_ctree, parent);

#warning are we freeing all lirc_code and lirc_config of the lists?
	kill_comments(cur_entry);
	g_free(cur_entry);

}
void add_item(void)
{
	GList *selection = GTK_CLIST(config_ctree)->selection;
	GtkCTreeNode *node = selection ? selection->data: NULL;
	GtkCTreeNode *parent;
	GtkCTreeNode *sibling = NULL;
	gchar *mode = NULL;

	struct lirc_config *lirc_config =
		gtk_object_get_data(GTK_OBJECT(config_ctree), "lirc_config");
	struct lirc_config_entry *entry = g_new0(struct lirc_config_entry, 1);
	GtkCTreeRow *row;
	gchar *col_text[5] = {
		"none",
		NULL,
		"*",
		"*",
		NULL,
	};
	new_comments(entry, lirc_com_TOTAL);
	entry->prog = g_strdup("none");
	entry->code = g_new0(struct lirc_code, 1);
	new_comments(entry->code, 1);
	entry->config = g_new0(struct lirc_list, 1);
	new_comments(entry->config, 1);
	entry->config->string = g_strdup("");

	if(node)
		row = GTK_CTREE_ROW(node);

	if(!node && !lirc_config){
		lirc_config = g_new0(struct lirc_config, 1);
		lirc_config->first = entry;
		fill_CTree(lirc_config);
		return;
	}else if(!node){
		parent=NULL;
	}else if(row->is_leaf){
		parent=row->parent;
		if(row->sibling)
			sibling = GTK_CTREE_NODE_NEXT(node);
		if(row->parent)
			mode = gtk_ctree_node_get_row_data(config_ctree,
							   row->parent);
	}else{
		mode = gtk_ctree_node_get_row_data(config_ctree, node);
		parent=node;
		if(row->children)
			sibling = row->children;
	}
	
	if(mode)
		entry->mode = g_strdup(mode);

	/* we don't want a select signal on the node before it has it's entry
	 * attached */
	gtk_signal_handler_block_by_func (GTK_OBJECT(config_ctree),
					  ctree_node_select, NULL);

	node = gtk_ctree_insert_node(config_ctree,
				     parent,
				     sibling,
				     col_text,
				     2,
				     NULL,
				     NULL,
				     NULL,
				     NULL,
				     TRUE,
				     FALSE);
	gtk_ctree_node_set_row_data (config_ctree, node, entry);

	/* restore selection callback the node is ready to be selected */
	gtk_signal_handler_unblock_by_func (GTK_OBJECT(config_ctree),
					  ctree_node_select, NULL);

	gtk_ctree_select (config_ctree, node);
	gtk_ctree_node_moveto (config_ctree, node, 0, 0, 0);
}
void rename_mode_cb(gchar *name, GtkCTreeNode *node)
{
	gchar *old_name;
	gtk_main_quit();
	if(name == NULL)
		return;

	gtk_ctree_node_set_text (config_ctree, node, 0, name);
	old_name=gtk_ctree_node_get_row_data(config_ctree, node);
	gtk_ctree_node_set_row_data(config_ctree, node, g_strdup(name));
	if(old_name)
		rename_mode_comments(old_name, name);
}
void rename_mode(GtkCTreeNode *node)
{
	GtkWindow *toplevel = GTK_WINDOW(
		gtk_widget_get_toplevel (GTK_WIDGET(config_ctree)));
	GtkWidget *dialog = gnome_request_dialog (FALSE,
						  "New name for this mode",
						  "New_mode",
						  0,
						  (GnomeStringCallback)
						  rename_mode_cb,
						  node,
						  toplevel);
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
	gtk_main();
	
#warning Does the dialog destroy automaticly?
	/*gtk_widget_destroy(dialog);*/
}
void add_mode(void)
{
	gchar *name = "New_Mode";
	char *text[4]= {
		name,
		NULL,
		NULL,
		NULL,
	};
	GtkCTreeNode *node = gtk_ctree_insert_node(config_ctree,
						   NULL,
						   NULL,
						   text,
						   10,
						   NULL,
						   NULL,
						   NULL,
						   NULL,
						   FALSE,
						   TRUE);
	rename_mode(node);

	name = gtk_ctree_node_get_row_data(config_ctree, node);

	if(name != NULL){
		new_mode_comments(name);
		gtk_ctree_select(config_ctree, node);
		add_item();
	} else {
		/* the user canceled the rename and we assume that he
		 * doesn't wan't the new mode*/
		gtk_ctree_remove_node(config_ctree, node);
	}
	
}
void fill_CTree(struct lirc_config *lirc_config)
{
	struct lirc_config_entry *entry=lirc_config->first;
	char *prev_mode=NULL;
	char *cur_mode=NULL;
	GtkCTreeNode *cur_parent=NULL;
	GtkCTreeNode *node=NULL;
	GList *known_modes = NULL;

	gtk_clist_freeze(GTK_CLIST(config_ctree));

	/* block selection callbacks while we play around with the widgets */
	gtk_signal_handler_block_by_func (GTK_OBJECT(config_ctree),
					  ctree_node_select, NULL);
	gtk_signal_handler_block_by_func (GTK_OBJECT(config_ctree),
					  ctree_node_unselect, NULL);

#warning should free all data related to the old lirc_config struct

	gtk_ctree_post_recursive (config_ctree, NULL, remove_node, NULL);

#if 0
	gtk_container_foreach (GTK_CONTAINER(config_ctree),
			       (GtkCallback)gtk_widget_destroy,
			       NULL);
#endif
	gtk_object_set_data(GTK_OBJECT(config_ctree), 
			    "lirc_config", lirc_config);

	while (entry){
		char **comments = get_comments(entry);
		gboolean new_parent=FALSE;
		gchar *col_text[5] = {
			NULL,
			NULL,
			"*",
			"*",
			NULL,
		};
		cur_mode = entry->mode;
		if(cur_mode != NULL && prev_mode == NULL){
			//entering new mode
			prev_mode = cur_mode = entry->mode;
			new_parent = TRUE;
			known_modes = g_list_append(known_modes, cur_mode);
		}
		if (cur_mode == NULL && prev_mode != NULL){
			//leaving a mode
			prev_mode = NULL;
			cur_parent = NULL;
		}
		if (cur_mode && prev_mode 
		    && (strcmp(cur_mode, prev_mode) !=0)){
			//changing mode
			prev_mode=cur_mode;
			new_parent = TRUE;
			cur_parent = NULL;
			known_modes = g_list_append(known_modes, cur_mode);
		}
		// begin entry

		if(entry->prog){
			col_text[0] = entry->prog;
		}
		if(comments[lirc_com_tag])
			col_text[1] = comments[lirc_com_tag];
		if(entry->code){
			if(entry->code->remote){
				col_text[2] = entry->code->remote;
			}
			if(entry->code->button){
				col_text[3] = entry->code->button;
			}
		}
		if(entry->rep != 0){
		}
		if(entry->config){
			col_text[4] = entry->config->string;
		}
		if(entry->change_mode){
		}
		if(entry->flags){
			if(entry->flags & once){}
			if(entry->flags & quit){}
			if(entry->flags & mode){}
			if(entry->flags & ecno){}
			if(entry->flags & startup_mode){}
		}
		if(new_parent){
			char *text[4]= {
				entry->mode,
				NULL,
				NULL,
				NULL,
			};
			cur_parent= gtk_ctree_insert_node(config_ctree,
							  NULL,
							  NULL,
							  text,
							  10,
							  NULL,
							  NULL,
							  NULL,
							  NULL,
							  FALSE,
							  FALSE);
			gtk_ctree_node_set_row_data(config_ctree, cur_parent,
						    g_strdup(entry->mode));
		}
		node = gtk_ctree_insert_node(config_ctree,
					     cur_parent,
					     NULL,
					     col_text,
					     2,
					     NULL,
					     NULL,
					     NULL,
					     NULL,
					     TRUE,
					     FALSE);
		gtk_ctree_node_set_row_data (config_ctree, node, entry);
		entry=entry->next;
	}

	if(known_modes!=NULL){
		gchar *text = gtk_editable_get_chars(GTK_EDITABLE(mode_entry),
						     0, -1);
		
		gtk_combo_set_popdown_strings(mode_combo, known_modes);
		gtk_entry_set_text(mode_entry, text);
		widget_changed_cancel(GTK_WIDGET(mode_entry));
		g_free(text);
		g_list_free(known_modes);
	}

	/* restore selection callbacks */
	gtk_signal_handler_unblock_by_func (GTK_OBJECT(config_ctree),
					  ctree_node_select, NULL);
	gtk_signal_handler_unblock_by_func (GTK_OBJECT(config_ctree),
					  ctree_node_unselect, NULL);

	gtk_clist_thaw(GTK_CLIST(config_ctree));
	gtk_clist_select_row(GTK_CLIST(config_ctree), 0, 0);


}

static void relink_lirc_config_func(GtkCTree *ctree, GtkCTreeNode *node,
				    gpointer arg)
{
	struct lirc_config_entry **prev_entry = arg;
	struct lirc_config_entry *entry;

	if (!GTK_CTREE_ROW(node)->is_leaf)
		return;

	entry = gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(node));
	
	if (*prev_entry == NULL){
		struct lirc_config *lirc_config =
			gtk_object_get_data(GTK_OBJECT(config_ctree),
					    "lirc_config");
		lirc_config->first = entry;
	} else
		(*prev_entry)->next = entry;

	*prev_entry = entry;
	entry->next = NULL;
}

void relink_lirc_config(void)
{
	struct lirc_config_entry *prev_entry = NULL;

	gtk_ctree_pre_recursive(config_ctree, NULL,
				relink_lirc_config_func, &prev_entry);
}
static void ctree_node_move(GtkCTree *ctree, GList *node,
			    GList *new_parent, GList *new_sibling,
			    gpointer user_data)
{
	GtkCTreeRow *row = GTK_CTREE_ROW(node);
	gboolean after = GPOINTER_TO_INT(user_data);
	static GtkCTreeNode *old_parent = NULL;
	static GtkCTreeNode *old_sibling = NULL;
	static gboolean sending_back = FALSE;

	if(sending_back)
		return;

	if (after) {
		/* Recursive modes are not allowed */
		if (new_parent && !GTK_CTREE_ROW(new_parent)->is_leaf
		    && !GTK_CTREE_ROW(node)->is_leaf) {
			sending_back = TRUE;
			gtk_ctree_move (config_ctree, GTK_CTREE_NODE (node),
					old_parent, old_sibling);
			sending_back = FALSE;
		} else if (row->parent){
			gchar *mode;
			struct lirc_config_entry *entry;
			entry = gtk_ctree_node_get_row_data(
				ctree, GTK_CTREE_NODE(node));
			mode = gtk_ctree_node_get_row_data(config_ctree,
							   row->parent);
			g_free(entry->mode);
			entry->mode = g_strdup(mode);
		}
	} else {
		old_parent = row->parent;
		old_sibling = row->sibling;
	}
}
void interface_apply_changes(void)
{
	GList *selection = GTK_CLIST(config_ctree)->selection;

	if(selection){
		/* make sure all changes are applyed to the data
		 * structures */
		ctree_node_unselect(config_ctree, selection->data, 0, NULL);
		ctree_node_select(config_ctree, selection->data, 0, NULL);
	}

	relink_lirc_config();
}

static gboolean map (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	/* we will be visible, owr notebook tab has been selected */
	return TRUE;
}
static gboolean unmap (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	/* we will not be visible, another notebook tab has been
	 * selected */
	interface_apply_changes();
	return TRUE;
}


void init_interface(GladeXML *xml)
{

	config_ctree = GTK_CTREE(glade_xml_get_widget(xml, "config_ctree"));

	/* connect signals for the ctree management */
	gtk_signal_connect(GTK_OBJECT(config_ctree),
			   "tree-select-row",
			   ctree_node_select,
			   NULL);
	gtk_signal_connect(GTK_OBJECT(config_ctree),
			   "tree-unselect-row",
			   ctree_node_unselect,
			   NULL);
	gtk_signal_connect(GTK_OBJECT(config_ctree),
			   "tree-move",
			   ctree_node_move, GINT_TO_POINTER(FALSE));
	gtk_signal_connect_after(GTK_OBJECT(config_ctree),
				 "tree-move",
				 ctree_node_move, GINT_TO_POINTER(TRUE));

	gtk_ctree_set_reorderable(config_ctree, TRUE);

	/* Get widgets for enabling/disabling*/
	editor_main_table = glade_xml_get_widget(xml, "editor_main_table");

	gtk_signal_connect(GTK_OBJECT(editor_main_table),
			   "map",
			   (GtkSignalFunc)map,
			   NULL);
	gtk_signal_connect(GTK_OBJECT(editor_main_table),
			   "unmap",
			   (GtkSignalFunc)unmap,
			   NULL);

	/* Get required widget pointers */
	about_dialog = glade_xml_get_widget(xml, "about-dialog");
	gnome_dialog_close_hides(GNOME_DIALOG(about_dialog), TRUE);
	glade_xml_signal_connect (xml, "on_about_activate", show_about_dialog);

	mode_entry = GTK_ENTRY(glade_xml_get_widget(xml, "mode_entry"));
	connect_changed_signal(GTK_WIDGET(mode_entry));

	prog_entry = GTK_ENTRY(glade_xml_get_widget(xml, "prog_entry"));
	connect_changed_signal(GTK_WIDGET(prog_entry));

	remote_entry = GTK_ENTRY(glade_xml_get_widget(xml, "remote_entry"));
	connect_changed_signal(GTK_WIDGET(remote_entry));

	button_entry = GTK_ENTRY(glade_xml_get_widget(xml, "button_entry"));
	connect_changed_signal(GTK_WIDGET(button_entry));

	config_entry = GTK_ENTRY(glade_xml_get_widget(xml, "config_entry"));
	connect_changed_signal(GTK_WIDGET(config_entry));

	tag_entry = GTK_ENTRY(glade_xml_get_widget(xml, "tag_entry"));
	connect_changed_signal(GTK_WIDGET(tag_entry));


	mode_combo = GTK_COMBO(glade_xml_get_widget(xml, "mode_combo"));
	prog_combo = GTK_COMBO(glade_xml_get_widget(xml, "prog_combo"));
	remote_combo = GTK_COMBO(glade_xml_get_widget(xml, "remote_combo"));
	button_combo = GTK_COMBO(glade_xml_get_widget(xml, "button_combo"));
	config_combo = GTK_COMBO(glade_xml_get_widget(xml, "config_combo"));

	repeat_spinbutton = GTK_SPIN_BUTTON(
		glade_xml_get_widget(xml, "repeat_spinbutton"));
	connect_changed_signal(GTK_WIDGET(repeat_spinbutton));

	once_checkbutton = GTK_TOGGLE_BUTTON(
		glade_xml_get_widget(xml, "once_checkbutton"));
	connect_changed_signal(GTK_WIDGET(once_checkbutton));

	quit_checkbutton = GTK_TOGGLE_BUTTON(
		glade_xml_get_widget(xml, "quit_checkbutton"));
	connect_changed_signal(GTK_WIDGET(quit_checkbutton));

	mode_checkbutton = GTK_TOGGLE_BUTTON(
		glade_xml_get_widget(xml, "mode_checkbutton"));
	connect_changed_signal(GTK_WIDGET(mode_checkbutton));

	startup_mode_checkbutton = GTK_TOGGLE_BUTTON(
		glade_xml_get_widget(xml, "startup_mode_checkbutton"));
	connect_changed_signal(GTK_WIDGET(startup_mode_checkbutton));

	button_list = GTK_LIST(glade_xml_get_widget(xml, "button_list"));
	config_list = GTK_LIST(glade_xml_get_widget(xml, "config_list"));

	setup_interface_list("config", xml);
	setup_interface_list("button", xml);

	glade_xml_signal_connect(xml, "on_delete_item", delete_item);
	glade_xml_signal_connect(xml, "on_add_item", add_item);
	glade_xml_signal_connect(xml, "on_add_mode", add_mode);

	init_file_read_write(xml);
	init_interface_lircd(xml);
	init_interface_text(xml);

	new_config_file();
}
