/*  lircrc-config pretends to be a GUI for LIRC's rc files.
    Copyright (C) 2001 Manuel Estrada Sainz <ranty@atdot.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

%{
#include <string.h>
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <lirc/lirc_client.h>
#include "parser.h"

void *yy_scan_string( const char *yy_str );
void yy_delete_buffer( void *b );

static void yyerror (char *s);
int yylex (void);
extern int yylineno;
extern guint yytext_pos;
extern void yyrestart(FILE *);
static void indent(GString *str, gint indent);
static void append_indented(GString *str, gchar *text, gint indent_level);
  
unsigned int lirc_flags(char *string);
  
static gchar **comments = NULL;
static gchar **mode_comments = NULL;
static gchar *current_remote = NULL;
  
static struct lirc_config *config;
static struct lirc_config_entry *entry=NULL;
static char *current_mode=NULL;
%}
  
/* BISON Declarations */
%union {
	int num;
	char *str;
}
%token DEC
%token QUOT_STRING
%token HEX
%token RVAL_STRING
%token ID

%token BeGIN //Can't use BEGIN as it is internaly used by lex
%token  PROG
%token  REMOTE
%token  BUTTON
%token  REPEAT
%token  CONFIG
%token  MODE
%token  FLAGS
%token END

//%type <addr> addr
//%type <addr> index
//%type <addr> value
%%
input:    group 
        | input group 
	| //Accept empty files
;
group:	  entry 
	| mode
;

entry:	  begin_entry {
		}
		values
	  end_entry {
		}
;
values:   value
	| values value
;
value:	  prog
	| remote
	| button
	| repeat
	| config
	| mode
	| flags
;

mode:	  begin_mode 
		entries 
	  end_mode 
;
entries:	  entry 
		| entries entry 
;
/***********
 * Entryes *
 ***********/
begin_entry:	BeGIN '\n' {
			g_assert(entry==NULL);
			entry = g_new0(struct lirc_config_entry, 1);
			comments = new_comments(entry, lirc_com_TOTAL);
			comments[lirc_com_entry] = $<str>1;
			if($<str>1){
				gchar *tag = strrchr($<str>1, '#');
				gchar *newline = strchr(tag, '\n');
				if(strncmp("#tag:", tag, 5)==0){
					if (newline) *newline='\0';
					comments[lirc_com_tag] =
							g_strdup(tag+5);
					*tag = '\0';
				}
			}
			
			if(config->first == NULL) {
				config->first = entry;
				config->next = entry;
			} else {
				config->next->next = entry;
				config->next = entry;
			}
			if(current_mode)
				entry->mode = g_strdup(current_mode);
		}
;

prog:	  PROG RVAL_STRING '\n'	{
			entry->prog = $<str>2;
			comments[lirc_com_prog] = $<str>1;
		}
;
remote:	  REMOTE RVAL_STRING '\n' {
			current_remote = $<str>2;
			comments[lirc_com_remote] = $<str>1;
		}
;
button:	  BUTTON RVAL_STRING '\n' {
			struct lirc_code *new_code = g_new0(struct lirc_code,
							    1);
			gchar **code_comments = new_comments(new_code, 1);

			new_code->remote = g_strdup(current_remote);
			new_code->button = $<str>2;
			code_comments[lirc_com_code_button] = $<str>1;
			if(comments[lirc_com_remote]){
				code_comments[lirc_com_code_remote] =
						comments[lirc_com_remote];
				comments[lirc_com_remote] = NULL;
			}
			if(entry->code == NULL)
				entry->code = new_code;
			else
				entry->next_code->next = new_code;

			entry->next_code = new_code;
		}
;
repeat:	  REPEAT RVAL_STRING '\n' {
			entry->rep = atoi($<str>2);
			g_free($<str>2);
			comments[lirc_com_repeat] = $<str>1;
		}

;
config:	  CONFIG RVAL_STRING '\n' {
			struct lirc_list *new_config = g_new0(struct lirc_list,
							      1);
			gchar **config_comments = new_comments(new_config, 1);

			new_config->string = $<str>2;
			*config_comments = $<str>1;
			
			if(entry->config == NULL)
				entry->config = new_config;
			else
				entry->next_config->next = new_config;

			entry->next_config = new_config;
		}
;
mode:	  MODE RVAL_STRING '\n' {
			entry->change_mode = $<str>2;
			comments[lirc_com_mode] = $<str>1;
		}

;
flags:	  FLAGS RVAL_STRING '\n' {
			entry->flags = lirc_flags($<str>2);
			g_free($<str>2);
			comments[lirc_com_flags] = $<str>1;
		}

;

end_entry:		END '\n' {
			g_free(current_remote);
			comments[lirc_com_end] = $<str>1;
			current_remote=NULL;
			entry = NULL;
			comments=NULL;
		}

;

/*********
 * Modes *
 *********/

begin_mode:	  BeGIN ID '\n' {
			g_assert(current_mode == NULL);
			mode_comments = new_mode_comments($<str>2);
			mode_comments[lirc_mode_com_begin] = $<str>1;

			current_mode = $<str>2;
		}
;

end_mode:	  END ID '\n' {
			g_assert(current_mode != NULL);
			if(strcmp(current_mode, $<str>2) != 0){
				g_print("\"%s\" doesn't match mode \"%s\"\n",
					$<str>2, current_mode);
				yyerror("parse error");
				return;
			}
				
			g_free($<str>2);
			g_free(current_mode);
			current_mode = NULL;

			mode_comments[lirc_mode_com_end] = $<str>1;
			mode_comments = NULL;
		}
;

%%

extern FILE *yyin;

/*
 * This converts the parsed data structures into a string
 * for debuging and to make sure I knew how to do it.
 *
 * It will also be usefull to write the modified configuration
 * once edided with the GUI.
 */
void config_2_g_string(GString *str, struct lirc_config *config)
{
	struct lirc_config_entry *entry=config->first;
	int indent_level = 0;
	char *prev_mode=NULL;
	char *cur_mode=NULL;

	reset_entry_text_pos();

	while (entry){
		gchar **comments = get_comments(entry);
		guint16 text_pos_start=0;

		cur_mode = entry->mode;
		if(cur_mode != NULL && prev_mode == NULL){
			//entering new mode
			char **mode_comments = get_mode_comments(cur_mode);
			prev_mode = cur_mode = entry->mode;
			if(mode_comments[lirc_mode_com_begin])
				append_indented(str,
					mode_comments[lirc_mode_com_begin],
					indent_level);
			indent(str, indent_level++);
			g_string_sprintfa(str,"begin %s\n", cur_mode);
		}
		if (cur_mode == NULL && prev_mode != NULL){
			//leaving a mode
			char **mode_comments = get_mode_comments(prev_mode);
			if(mode_comments[lirc_mode_com_end])
				append_indented(str,
					mode_comments[lirc_mode_com_end],
					indent_level);
			indent(str, --indent_level);
			g_string_sprintfa(str,"end %s\n", prev_mode);
			prev_mode = NULL;
		}
		if (cur_mode && prev_mode && (strcmp(cur_mode, prev_mode) !=0)){
			//changing mode
			char **mode_comments = get_mode_comments(prev_mode);
			indent(str, --indent_level);
			if(mode_comments[lirc_mode_com_end])
				append_indented(str,
					mode_comments[lirc_mode_com_end],
					indent_level);
			g_string_sprintfa(str,"end %s\n", prev_mode);
			
			mode_comments = get_mode_comments(cur_mode);
			if(mode_comments[lirc_mode_com_begin])
				append_indented(str,
					mode_comments[lirc_mode_com_begin],
					indent_level);
			g_string_sprintfa(str,"begin %s\n", cur_mode);
			indent(str, indent_level++);
			prev_mode=cur_mode;
		}
		// begin entry

		if(comments[lirc_com_entry])
			append_indented(str, comments[lirc_com_entry],
				indent_level);

		text_pos_start = str->len;

		if(comments[lirc_com_tag]){
			indent(str, indent_level);
			g_string_sprintfa(str, "#tag:%s\n",
					comments[lirc_com_tag]);
		}
		indent(str, indent_level++);
		g_string_sprintfa(str,"begin\n");

		if(entry->prog){
			if(comments[lirc_com_prog])
				append_indented(str, comments[lirc_com_prog],
					indent_level);
			indent(str, indent_level);
			g_string_sprintfa(str,"prog   = %s\n", entry->prog);
		}
		entry->next_code = entry->code;
		if(comments[lirc_com_remote])
			append_indented(str, comments[lirc_com_remote],
				indent_level);
		while(entry->next_code){
			gchar **code_com = get_comments(entry->next_code);
			if(code_com[lirc_com_code_remote])
				append_indented(str,
						code_com[lirc_com_code_remote],
						indent_level);
			if(entry->next_code->remote){
				indent(str, indent_level);
				g_string_sprintfa(str,"remote = %s\n",
						entry->next_code->remote);
			}
			if(code_com[lirc_com_code_button])
				append_indented(str,
						code_com[lirc_com_code_button],
						indent_level);
			if(entry->next_code->button){
				indent(str, indent_level);
				g_string_sprintfa(str,"button = %s\n",
						entry->next_code->button);
			}
			entry->next_code=entry->next_code->next;
		}
		if(entry->rep != 0 || comments[lirc_com_repeat]){
			if(comments[lirc_com_repeat])
				append_indented(str, comments[lirc_com_repeat],
					indent_level);
			indent(str, indent_level);
			g_string_sprintfa(str,"repeat = %d\n", entry->rep);
		}
		entry->next_config = entry->config;
		while(entry->next_config){
			gchar **conf_com = get_comments(entry->next_config);
			if(*conf_com)
				append_indented(str, *conf_com, indent_level);
			indent(str, indent_level);
			g_string_sprintfa(str,"config = %s\n",
						entry->next_config->string);
			entry->next_config=entry->next_config->next;
		}
		if(entry->change_mode){
			if(comments[lirc_com_mode])
				append_indented(str, comments[lirc_com_mode],
					indent_level);
			indent(str, indent_level);
			g_string_sprintfa(str,"mode   = %s\n",
						entry->change_mode);
		}
		if(entry->flags || comments[lirc_com_flags]){
			if(comments[lirc_com_flags])
				append_indented(str, comments[lirc_com_flags],
					indent_level);
			indent(str, indent_level);
			g_string_sprintfa(str,"flags  =");
			if(entry->flags & once)
				g_string_sprintfa(str," once");
			if(entry->flags & quit)
				g_string_sprintfa(str," quit");
			if(entry->flags & mode)
				g_string_sprintfa(str," mode");
			if(entry->flags & ecno)
				g_string_sprintfa(str," ecno");
			if(entry->flags & startup_mode)
				g_string_sprintfa(str," startup_mode");
			g_string_sprintfa(str,"\n");
		}
		// end entry
		indent(str, --indent_level);
		g_string_sprintfa(str,"end\n");

		set_entry_text_pos(entry, text_pos_start, str->len);

		entry=entry->next;
	}
	if (cur_mode != NULL){
		//leaving a mode
		char **mode_comments = get_mode_comments(prev_mode);
		if(mode_comments[lirc_mode_com_end])
			append_indented(str,
				mode_comments[lirc_mode_com_end],
				indent_level);
		indent(str, --indent_level);
		g_string_sprintfa(str,"end %s\n", prev_mode);
		prev_mode = NULL;
	}

}
struct lirc_config *parse_config_file (const char *file)
{
	FILE *file_p;
#if YYDEBUG != 0
	extern int yydebug;
	yydebug=1;
#endif
	if((file_p = fopen(file, "r"))==NULL){
		perror("fopen");
		return NULL;
	}
	yyrestart(file_p);
	yylineno=1;
	config = g_new0(struct lirc_config, 1);
	yyparse ();
	fclose (yyin);
	/* config may be null if the parsing failed */
	return config;
}

struct lirc_config *parse_config_text (const char *str)
{
	void * buffer;
#if YYDEBUG != 0
	extern int yydebug;
	yydebug=1;
#endif
	buffer = (void*) yy_scan_string(str);
	if(buffer==NULL){
		perror("yy_scan_bytes");
		return NULL;
	}
	yylineno=1;
	config = g_new0(struct lirc_config, 1);
	yyparse ();
	yy_delete_buffer(buffer);
	/* config may be null if the parsing failed */
	return config;
}

static void yyerror (s)  /* Called by yyparse on error */
     char *s;
{
#warning should find a better way of reporting errors
	/* free config so they know an error happened */
	g_free(config);
	config=NULL;

	/* cleanup and make sure things are reasonable for the next parse */
	g_free(current_remote);
	current_remote = NULL;
	g_free(current_mode);
	current_mode = NULL;
	g_free(entry);
	entry = NULL;

	printf ("%s in line %d\n", s, yylineno);
}
static void indent(GString *str, gint level)
{
	int i=0;
	for (i=0; i<level; i++){
		g_string_sprintfa(str,"\t");
	}
}
static void append_indented(GString *str, gchar *text, gint indent_level)
{
	gchar *next = strchr(text, '\n');
	for(; next != NULL; next = strchr(text, '\n')){
		char aux = *next;
		*next='\0';
		indent(str, indent_level);
		g_string_sprintfa(str, "%s\n", text);
		text = next+1;
		*next=aux;
	}
	if(*text != '\0'){
		indent(str, indent_level);
		g_string_sprintfa(str, "%s\n", text);
	}
}
